'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _toConsumableArray = require('@babel/runtime/helpers/toConsumableArray');
var compilerDom = require('@vue/compiler-dom');
var parser = require('@babel/parser');
var traverse$1 = require('@babel/traverse');
var generator = require('@babel/generator');
var loaderUtils = require('loader-utils');
var serializeJavascript = require('serialize-javascript');

var genImportDeclaration = function genImportDeclaration(name, path) {
  var normalizePath = path.replace(/\\/g, '/');
  var value = "import ".concat(name, " from '").concat(normalizePath, "'");
  return value;
};
var genMixinImport$1 = function genMixinImport(options) {
  var pathKeys = Object.keys(options).filter(function (key) {
    return key !== 'custom' && typeof options[key] === 'string';
  });
  var mixinImport = {};
  if (!pathKeys.length) {
    return {
      mixinImport: mixinImport,
      mixinImportNodes: []
    };
  }
  pathKeys.forEach(function (key) {
    var mixinKey = "".concat(key, "Mixin");
    mixinImport[mixinKey] = genImportDeclaration(mixinKey, options[key]);
  });
  var mixinImportAst = parser.parse(Object.values(mixinImport).join('\n'), {
    sourceType: 'module'
  });
  var mixinImportNodes = mixinImportAst.program.body;
  return {
    mixinImport: mixinImport,
    mixinImportNodes: mixinImportNodes
  };
};
var genMixinCustom$1 = function genMixinCustom(options) {
  if (options.custom) {
    var customMixinAst = parser.parse("const customMixin = ".concat(options.custom), {
      sourceType: 'module'
    });
    return customMixinAst.program.body;
  }
  return [];
};
var genNewMixinNode$1 = function genNewMixinNode() {
  var ast = parser.parse("const customMixin = { mixins: [] }", {
    sourceType: 'module'
  });
  return ast.program.body[0].declarations[0].init.properties[0];
};
var generate$1 = {
  genMixinImport: genMixinImport$1,
  genMixinCustom: genMixinCustom$1,
  genNewMixinNode: genNewMixinNode$1
};

var toString = function toString(val) {
  return Object.prototype.toString.call(val);
};
var isFunction = function isFunction(val) {
  return toString(val).slice(8, -1) === 'Function';
};
var isObject = function isObject(val) {
  return toString(val).slice(8, -1) === 'Object';
};
var isConstructorFn = function isConstructorFn(val) {
  var PropTypeStr = ['function String() { [native code] }', 'function Number() { [native code] }', 'function Boolean() { [native code] }', 'function Array() { [native code] }', 'function Object() { [native code] }', 'function Date() { [native code] }', 'function Function() { [native code] }', 'function Symbol() { [native code] }'];
  var validate = function validate(val) {
    return isFunction(val) && PropTypeStr.includes(val.toString());
  };
  var genConstructorStr = function genConstructorStr(val) {
    return val.toString().match(/^function\s([^(]+)/g)[0].slice(9);
  };
  if (Array.isArray(val)) {
    if (val.every(validate)) {
      return "__ConstructorFn([".concat(val.map(genConstructorStr).join(', '), "])");
    }
  } else {
    if (validate(val)) {
      return "__ConstructorFn(".concat(genConstructorStr(val), ")");
    }
  }
  return;
};
var removeConstructorFnTag = function removeConstructorFnTag(str) {
  return str.replace(/['"]__ConstructorFn\(([^)]+)\)['"]/g, '$1');
};

// 移除属性名两端的双引号
var removePropQuotes = function removePropQuotes(str) {
  return str.replace(/"([\w\d_]+)":/g, '$1:');
};
var stringify_1$1 = function stringify_1(obj) {
  var handleConstructorVal = function handleConstructorVal(obj) {
    var str;
    Object.keys(obj).forEach(function (key) {
      if (str = isConstructorFn(obj[key])) {
        obj[key] = str;
      }
      if (isObject(obj[key])) {
        handleConstructorVal(obj[key]);
      }
    });
  };
  handleConstructorVal(obj);
  var res = serializeJavascript(obj, {
    space: 2
  });
  return removePropQuotes(removeConstructorFnTag(res));
};
var stringify$1 = {
  stringify: stringify_1$1
};

var parse = compilerDom.parse;
var traverse = traverse$1["default"];
var generate = generator["default"];
var getOptions = loaderUtils.getOptions;
var genMixinCustom = generate$1.genMixinCustom,
  genMixinImport = generate$1.genMixinImport,
  genNewMixinNode = generate$1.genNewMixinNode;
var stringify = stringify$1.stringify;
var handler = function handler(source, sourceMaps) {
  var _scriptAst$program$bo;
  var options = getOptions(this);
  if (!Object.keys(options).length) return source;
  var sourceAST = parse(source);
  var script = sourceAST.children.find(function (v) {
    return v.tag === 'script';
  });
  var _ref = [script.loc.start.offset, script.loc.end.offset],
    startTag = _ref[0],
    endTag = _ref[1];
  var scriptTag = source.slice(startTag, endTag);
  var scriptContent = scriptTag.replace(/^<script>|<\/script>$/g, '');
  var start = startTag + '<script>'.length;
  var end = endTag - '</script>'.length;

  // custom 配置
  var customMixinNodes = genMixinCustom(options);
  // import mixin 配置
  var mixinImportInfo = genMixinImport(options);
  var scriptAst = parser.parse(scriptContent, {
    plugins: ['jsx'],
    sourceType: 'module'
  });
  var newImportNodes = mixinImportInfo.mixinImportNodes;
  traverse(scriptAst, {
    ImportDeclaration: function ImportDeclaration(path) {
      // 收集所有的 import 节点
      scriptAst.program.body = scriptAst.program.body.filter(function (node) {
        return node !== path.node;
      });
      newImportNodes.push(path.node);
    },
    ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
      var _mixins$value$element;
      var properties = path.node.declaration.properties;
      // 既没有 customMixin 也没有从外部引入 importMixin
      if (!customMixinNodes.length && !mixinImportInfo.mixinImportNodes.length) return;
      var mixins = properties.find(function (property) {
        return property.key.name === 'mixins';
      });
      // 是否有 mixins 配置
      if (!mixins) {
        mixins = genNewMixinNode();
        properties.unshift(mixins);
      }
      var newMixins = Object.keys(mixinImportInfo.mixinImport);
      if (customMixinNodes.length) {
        newMixins.push('customMixin');
      }
      var genMixinsElements = newMixins.map(function (key) {
        return {
          type: 'Identifier',
          name: key
        };
      });
      (_mixins$value$element = mixins.value.elements).push.apply(_mixins$value$element, _toConsumableArray(genMixinsElements));
    }
  });
  newImportNodes.push.apply(newImportNodes, _toConsumableArray(customMixinNodes));
  // 插入 mixin 节点
  (_scriptAst$program$bo = scriptAst.program.body).unshift.apply(_scriptAst$program$bo, _toConsumableArray(newImportNodes));
  var newScript = generate(scriptAst).code;
  var newSource = source.slice(0, start) + "\n".concat(newScript, "\n") + source.slice(end);
  return newSource;
};
var src = handler;
var stringify_1 = stringify;
src.stringify = stringify_1;

exports.default = src;
exports.stringify = stringify_1;
